# Costume inspect, to reduce noise
# 
# Instance variables of classes starting with Async or RSMP are shown
# with only their class name and object id, which reduces output,
# especially for deep object structures.
# Additionally, a list of variables to shown in short format can be passed.
#
# The short form is generated by using to_s() insted of inspect()
#
# Array#to_s and Hash#to_s usually show items, but here we show just number
# of items, when the short form is requested.

module RSMP
  module Inspect

    def inspector *short_items
      instance_variables.map do |var_name|
        var = instance_variable_get(var_name)
        class_name = var.class.name

        short = short_items.include?(var_name) ||
                   class_name.start_with?('Async') ||
                   class_name.start_with?('RSMP')

        if short
          if var.is_a? Array
            "#{var_name}: #<#{class_name}:#{class_name.object_id}, #{var.size} items>"
          elsif var.is_a? Hash
            "#{var_name}: #<#{class_name}:#{class_name.object_id}, #{var.size} items>"
          else
            "#{var_name}: #{var.to_s}"
          end
        else
          "#{var_name}: #{var.inspect}"
        end
      end.join(', ')
    end

    # override this if you want additional variable to be shown in the short format,
    # or ottherwise change the inspect format
    def inspect
      "#<#{self.class.name}:#{self.object_id}, #{inspector}>"
    end

  end
end